######################################################
######################################################
###  BASIC INTRODUCTION TO R 
######################################################
######################################################
# The R interface is a just that - a basic interface. In order to obtain R, you must first download the software from the link provided here: https://www.r-project.org/. Then, in order to use the software, you must load a dataset and use packages and commands in order to tell the software how to evaluate and handle the data. Each package is open sourced and contains different tools in order to implement all sorts of data manipulation techniques









######################################################
######## Packages, Libraries, Loading Data, and Viewing Data
######################################################

# 1. Install packages for use.

# Example: The foreign package and library allows you to load SPSS and STATA dataset file formats. You will want to load dependency packages also just in case the package/commands needs the support of others to work.
install.packages("foreign", dependencies=TRUE)
# EXAMPLE: The foreign package and library allows you to load SPSS and STATA datasets.

# 2. Loading a package for use.
library(foreign)

# 3. Loading the dataset.

# Example: Loading the 2014 European Social Survey Sweden dataset in STATA format (.dta format) that is provided on my webpage. When loading the dataset, the code here specified that factors should remain numerical. Therefore, if a variable was say gender (i.e. men and women), the variable will be coded as a number, such as 1 for men 2 for women. Note: I named the dataset "dat" here.
dat <- read.dta(file.choose(), convert.factors=FALSE)

# 4. Additional examples for different data formats:

# Example: Reading an SPSS file (.sav format) into R
dat <- read.spss(file.choose(), convert.factors=FALSE)

# Example: If you want to read an excel file (.xls or .xlsx format) into R you will need a different package. The code is below.
install.packages(gdata, dependencies=TRUE)
library(gdata)
dat <- read.xls(file.choose())

# 5. Exploring the Dataset.

# Looking at the names of the variables in the dataset.
names(dat)

# If we wanted to know the number of rows we have in the dataset (i.e. observations) we could use the "nrow" function. Here, we have 1791 Swedish Survey Respondents
nrow(dat)

# If we wanted to get a quick snapshat of the structure of the variables. We could simply summarize the entire dataset.
summary(dat)

# Additional Note: Help with a Library, Command, or Function

# Example:  If you would like to find out what function a command performs. A help file can be accessed by typing ?? and the code. If we wanted to know what function the "names" command performed, we could type the command below.
??names









######################################################
######## Understanding the Structure of Variables and Recoding
######################################################

# 1. Basic Exploration of Variables

# Variables are almost never in a readily useable format. Usually, there are missing values and people that did not answer questions. Therefore, the first step to any analysis is exploring the structure of the variables you want to use and making sure that they are accurately depicting the measurement scales intended.

# EXAMPLE: Let us pretend that the thing I ultimately want to explore is how Swedes view themselves ideologically (left - 0 to right - 10). In the survey, the codebook indicates that the variable "lrscale" provides this information. I can view a summary of this information by telling R that I want a summary of the lrscale variable in the "dat" dataset  that I loaded with the command below. Note: The dataset I am loading is titled "dat" and the variable in the dataset is "lrscale".
summary(dat$lrscale)

# Discussion: The output indicates that the minimum value for the variable is 0, median is 5, mean is 8.277 and max is 88. Does this make sense? No. Remember, the left-right ideology variable is measured from 0-10. Therefore, we need to continue our exploration.

# Example: You could also make a table of the variable in order to get a better view at the categories of the variable. 
table(dat$lrscale)

# Discussion: As you can see, there are two categories "77" and "88" that do not fit in our 0-10 scale. If we look in the codebook, 77 indicates ``Refusal" and 88 indicates "Don't know". In order to successfully move on and perform an analysis, we will need to recode this variable in order to remove the categories. 

# 2. Recoding Variables - Numerical

# In most instances, we might want to recode data so that these people do not exist in our analysis or "Not Applicable (NA)". For recoding variables, the "car" package provides us tool to do this type of recoding.
install.packages("car", dependencies=TRUE)
library(car)

# Example: As indicated before, it is necessary to recode the categories/numbers ``77" and ``88" as NA so that we can perform meaningful statistical tests on the data.

dat$ideology <- recode(dat$lrscale, "77=NA; 88=NA")
summary(dat$ideology)

# Discussion: After recoding the variable, the summary of the variable indicates that we now have a minimum value of 0, max of 10, mean of 5.047, and 70 observations have been set to NA. As you can see, the new mean makes much more sense theoretically.

# 3. Recoding Variables - Categorical

# Another situation that might arise is that R might treat a categorical variable as numeric in ways that we do not wish. For example, a variable that explores profession might have several categories that cannot be ranked in a meaningful way numerically. Therefore, the variable should be recoded as a factor.

# Example:  For simplicity sake, here we are recoding the gender variable ("gndr") to be categorical instead of numeric. In addition, if respondents refused to answer, we could do as we did above and recode other values to NA. The codebook indicates that for the variable a 1 indicates male and a 2 indicates female.

table(dat$gndr)
dat$gender <- recode(dat$gndr, "1='Man'; 2='Woman'")
dat$gender <- as.factor(dat$gender)
table(dat$gender)









######################################################
######## Graphically Displaying Variables
######################################################

# 1. Bar Chart

# Example: For categorical variables, the best mode of presentation is usually providing a simple bar chart. Here, we provide a bar chart that presents the frequencies for our newly created gender variable.
barchart(dat$gender, col="black")

# 2. Density Plot

# Example: For continuous numerical variables, a density plot usually provides a better snapshot. Here, we create two density plots. One density plot for political ideology and the other for political ideology while splitting up the sample by gender. 
densityplot(dat$ideology)

install.packages("ggplot2", dependencies=TRUE)
library(ggplot2)
ggplot(dat, aes(x=ideology)) + geom_density(aes(group=gender, colour=gender)) 









######################################################
######## Bivariate Analysis
######################################################
# 1. Simple Bivariate Correlation

# When starting our empirical bivariate analysis, we might simply want to know how correlated two variables are with each other. In other words, we are exploring how much of the variance that exists on one variable is dependent on the variation of another variable. Importantly, to perform a simple test of this nature, we must have two numeric variables.

# Example:  Let us pretend that I think political ideology in Sweden is dependent at least to some extent on the age of the respondent. I could perform a simply correlation test that explores whether these two variables are correlated.
# Note: First, I must recode the age variable using the year born variable "yrbrn".

table(dat$yrbrn)
dat$age <- recode(dat$yrbrn, "7777=NA")
dat$age <- 2014 - dat$age
table(dat$age)

cor(dat$ideology, dat$age, use = "pairwise.complete.obs")

# Discussion: The result is .0515 for the correlation test between political ideology and age. The interpretation is that around 5.15\% of the variation in political ideology between respondents is explained by age. The correlation relationship is quite weak.

# 2. T-Test

# When we have a continuous variable and a categorical variable, we might want to test whether the continuous variable's is statistically different based on the categories of our categorical variable.

t.test(dat$ideology~dat$gender)

# Discussion: Remember, our statistical significance cutoff is a p-value of .05 in most research. The p-value indicates the level of confidence that we have the correct sample. Here, we have a p-value of .01 (rounding). The value indicates that if we took 100 samples we would correctly have the correct sample 99 times. Here, if we guessed that gender had a statistical impact on ideology we would be correct 99/100 times. In particular, we can see that women are statistically more liberal. 

# 3. Cross Tabulation

# When we have two categorical variables we might want to test whether the categories of the variables have a relationship with one another. The easiest way to test this relationship is to perform a cross tabulation. 

# Example: Let us pretend that I think there is a relationship between gender and  unemployment in the last 12 months. I could test whether these two categorical variables have a relationship. 
# Note: I need to recode the "uemp12m" variable that asks about unemployment in the last 12 months.

table(dat$uemp12m)
dat$unemp <- recode(dat$uemp12m, "1='Yes'; 2='No'; else=NA")
table(dat$unemp)

install.packages("gmodels", dependencies=TRUE)
library(gmodels)
CrossTable(dat$unemp, dat$gender, chisq=TRUE)

# 4. Bivariate Regression analysis}
#The most useful way to test whether two variables are related is to estimate a regression model.

# Example: Let us say I wanted to calculate the precise effect that gender has on political ideology. Since political ideology is theoretically continuous, I could estimate an OLS linear regression model. 

mod <- lm(ideology ~ gender, data=dat)
summary(mod)

# Discussion: There are a few aspects of the model output to explore. First, the intercept. The intercept tells us the value of the dependent variable (political ideology) when all of the independent variables are at a value of zero (gender). Here, the zero category is ``man". Therefore, when gender is male the mean political ideology value is 5.19. Next, we can explore the coefficient estimate for the gender variable. Since the p-value is less than .05, the variable is significant.  When the gender of a respondent is female there is a decrease of -.28 in political ideology. The drop in political ideology is a little over a 1/4 of a point. Finally, we could look at model fit. How well does our model explain the variation in political ideology between respondents. The R-squared statistic provides this information. The value is .003. The value indicates that our model only explains around .3\% of the variation in political ideology.

# 5. Calculating and Plotting Variable Effects

After we have estimated a regression model, we have the possibility of plotting the precise effect that a significant independent variable has on the dependent variable.

# Example: Here, I want to plot the predictions for how gender effect political ideology.

install.packages("effects", dependencies=TRUE)
library(effects)

eff <- effect("gender", mod, default.levels=100)

effect <- print(plot(eff, rescale.axis=F, rug=FALSE, xlab="Gender", ylab="Political Ideology", main=" The Effect of Gender on Political Ideology"))









######################################################
######## Multivariate Analysis
######################################################

# 1. Multivariate OLS Linear Regression

# It is most assuredly unreasonable to assume that only one independent variable impacts a dependent variable of interest. In fact, there are usually multiple variables that impact one dependent variable. In order to account for this type of relationship, we can estimate a model with several independent variables much like how we did prior with the bivariate OLS linear regression. 

# Example: I want to know which variables have a statistical relationship with political ideology in Sweden.
#Note: This example requires the recoding of several variables. The code is provided in the script file.

summary(dat$edulvlb)
dat$education <- recode(dat$edulvlb, '0 = 1; 113 = 2; 129 = 3; 212 = 4; 213 = 5; 221 = 6; 222 = 7; 223 = 8; 229 = 9; 311 = 10; 312 = 11; 313 = 12; 321 = 13; 322 = 14; 323 = 15; 412 = 16; 413 = 17; 421 = 18; 422 = 19; 423 = 20; 510 = 21; 520 = 22; 610 = 23; 620 = 24; 710 = 25; 720 = 26; 800 = 27; else=NA')  
summary(dat$education)

summary(dat$hinctnt)
dat$income <- recode(dat$hinctnt, '77=NA; 88=NA; 99=NA')
summary(dat$income)

mod1 <- lm(ideology ~ age + gender + income + education, data=dat)
summary(mod1)

# Discussion: If you look at the p-values for our variables you will notice that there are three statistically significant variables that have a relationship with political ideology (age, gender, income). We already talked about the impact of gender. However, age and income have the opposite relationship. In particular, as you increase in age and income you identify at higher values of political ideology (meaning, you are more conservative). Notice, our R-squared statistic indicates that this model explains about 4\% of the variation of our dependent variable.  

# 2. Multivariate Logit Regression

# The example above was regression estimation for when the dependent variable of interest is a continuous numerical variable. However, many times in social science our variable of interest is a binary categorical variable. In order to explore this type of dependent variable, we must estimate a logistic regression. 

# Example: Let me pretend I have a strong theory to assume that Sweden Democrat voters are completely different from all other voters. Therefore, I might want to estimate a model that simply explores the variables that impact voting for the Sweden Democrats over all other parties. I would recode the vote choice variable so that voting for SD is a 1 and voting for all other parties is a 0. 

table(dat$prtvtbse)
dat$sdvote <- recode(dat$prtvtbse, "10=1; 66=NA; 77=NA; 88=NA; else=0")
table(dat$prtvtbse)

mod2 <- glm(sdvote ~ age + gender + income + education, data=dat, family=binomial) 
summary(mod2)

# Discussion: It is important to note that regression coefficients are uninterpretable when running logistic regression in terms of substantive numerical values. However, statistical significance and coefficient directionality can provide us with useful insights into findings. The only variable in our model that is statistically significant is the education variable. In particular, Sweden Democrat voters have a statistically lower level of education than all other voters. Since the coefficient direction is negative, it means that increases in education lead to decreases in the probability of voting for Swedish Democrats.

# 3. Calculating and Plotting Variable Effects}

#Since coefficients are difficult to interpret when estimating logistic regression, plotting predicted probabilities provides us with a useful way of exploring the substantive impact that an independent variable has on the dependent variable. \\

# Example:  Using the last multivariate model that was estimated, let us plot the effect that education has on the probability of voting for the Sweden Democrats.

eff2 <- effect("education", mod2, default.levels=100)

effect2 <- print(plot(eff2, rescale.axis=F, rug=FALSE, xlab="Education Level", ylab="Probability of Voting for Sweden Democrats", main=""))






save.image("/Users/hanse095/Dropbox/Fall 2017/Methods/Introduction to R/RIntroTutorial.RData")
load("/Users/hanse095/Dropbox/Fall 2017/Methods/Introduction to R/RIntroTutorial.RData")